function [YM,YP,C,D,S,W,ZM,ZP,E,F]=fuzzypca(XM,XP,A,B,q);
% function [YM,YP,C,D,S,W,ZM,ZP,E,F]=fuzzypca(XM,XP,A,B,q)
% PRINCIPAL COMPONENT ANALYSIS OF FUZZY DATA
% Inputs and outputs are trapezoidal fuzzy numbers  
%
% Inputs :
%		(XM,XP,A,B) : fuzzy input data (kernel=[XM;XP], support= [XM-A;XP+B])
%        XM, XP, A and B are all of size (n,d), n = number of input patterns, 
%                 d = input dimension
%        q : number of hidden units ( = dimension of reduced representation space)
%
% Outputs :
% 		(YM,YP,C,D) : fuzzy hidden layer outputs (kernel=[YM;YP], support= [YM-C;YP+D])
%                     all of size (n,q)
%		S	    : reconstruction errors (size (n,1))
%		W	    : weight matrix (size (d,q))
%	    (ZM,ZP,E,F) : Reconstructed outputs
%
% Uses : harris.m (optimization procedure), coutdiabflou1.m (error and gradient computation)

[n,d]=size(XM);
X{1}=XM-A;
X{2}=XM;
X{3}=XP;
X{4}=XP+B;

% OPTIMIZATION
W = 0.1*randn(d,q);
param=reshape(W,d*q,1);
options(1) = 1;
options(2) = 10000;
options(3) = 1e-3; 
options(4) = 100;
P = harris('coutdiabflou1',param,options,X,q);
W = reshape(P,d,q);

% PROPAGATION
YM = XP*(W.*(W<0)) + XM*(W.*(W>0));
YP = XM*(W.*(W<0)) + XP*(W.*(W>0));
C = -B*(W.*(W<0)) + A*(W.*(W>0));
D = -A*(W.*(W<0)) + B*(W.*(W>0));

ZM = YP*(W'.*(W'<0)) + YM*(W'.*(W'>0));
ZP = YM*(W'.*(W'<0)) + YP*(W'.*(W'>0));
E = -D*(W'.*(W'<0)) + C*(W'.*(W'>0));
F = -C*(W'.*(W'<0)) + D*(W'.*(W'>0));

% RECONSTRUCTION ERROR

S = (ZM-XM).^2 +  (ZP-XP).^2 + (ZM-E-XM+A).^2 + (ZP+F-XP-B).^2;
S=sum(S,2);

